<?php namespace Helper\PostsApi;
use stdClass;
class PostsApi {
 
  private $apiHostname;
  private $apiKey;
  private $apiToken;

  /* Construct Methods        */
  /***************************/

  public function __construct($argv = null) {
    if(isset($argv)) {
      $this->apiHostname = $argv["apiHostname"];
      $this->apiKey = $argv["apiKey"];
      $this->apiToken = $argv["apiToken"];
    }
  }

  /* Set Methods              */
  /***************************/
  
  /**
   * Set Api Hostname
   * @param string  $apiHostname   posts hostname
   */
  public function setApiHostname($apiHostname) {
    $this->apiHostname = $apiHostname;
  }
  /**
   * set ApiKey
   * @param string  $apiKey    posts apikey
   */
  public function setApiKey($apiKey) {
    $this->apiKey = $apiKey;
  }
  /**
   * Set ApiToken
   * @param string  $apiToken   posts apitoken
   */
  public function setApiToken($apiToken) {
    $this->apiToken = $apiToken;
  }

  /* Post Api - Core Methods */
  /***************************/
  
  /**
   * Get all Collections info
   * @return Return a Collection list
   */
  public function getCollections() {
    $endpoint = $this->apiHostname."collections";
    $query = http_build_query(array('apikey'=>$this->apiKey,'apitoken'=>$this->apiToken));
    $url = $endpoint."?".$query;
    
    return $this->restRequest($url);
  } 

  /**
   * Get Collection info
   * @return Show information about a Collection
   */
  public function getCollection($collection) {
    $endpoint = $this->apiHostname."collection/".$collection;
    $query = http_build_query(array('apikey'=>$this->apiKey,'apitoken'=>$this->apiToken));
    $url = $endpoint."?".$query;
    
    return $this->restRequest($url);
  } 

  /**
   * Get all dossiers from collection
   * @param  int       $collection   collection id
   * @return Return a Dossier list inside a Collection
   */
  public function getDossiers($collection) {
    $endpoint = $this->apiHostname."collection/".$collection."/dossiers";
    $query = http_build_query(array('apikey'=>$this->apiKey,'apitoken'=>$this->apiToken));
    $url = $endpoint."?".$query;
    
    return $this->restRequest($url);
  } 

  /**
   * Get info from dossier
   * @param  int       $collection    Collection id
   * @param  int       $dossier       Dossier id
   * @return Show information about a Dossier inside a Collection
   */
  public function getCollectionDossier($collection,$dossier) {
    $endpoint = $this->apiHostname."collection/".$collection."/dossier/".$dossier;
    $query = http_build_query(array('apikey'=>$this->apiKey,'apitoken'=>$this->apiToken));
    $url = $endpoint."?".$query;
    
    return $this->restRequest($url);
  } 

  /**
   * Get posts from dossier
   * @param  int    $collection Collection id
   * @param  int    $dossier    Dossier id
   * @param  string $ids        Query parameters
   * @return Return a Post lists inside a Dosseir Collection
   */
  public function getDossierPosts($collection,$dossier,$perPage=null,$page=null,$ids=null,$iso) {
    $endpoint = $this->apiHostname."collection/".$collection."/dossier/".$dossier."/posts";
    $query = http_build_query(array('apikey'=>$this->apiKey,'apitoken'=>$this->apiToken));
    if(isset($ids))
      $query.= "&ids=".$ids;

    if(isset($perPage) && isset($page))
      $query.= "&per_page=".$perPage."&page=".$page;
    else 
      if(isset($perPage)) $query.= "&per_page=".$perPage;
      else if(isset($page)) $query.= "&page=".$page;

    $query.="&iso=".$iso;
    $url = $endpoint."?".$query;
    return $this->restRequest($url);
  }

  /**
   * Get post data
   * @param  int    $collection Collection id
   * @param  int    $dossier    Dossier id
   * @param  int    $post       Post id
   * @param  string $ids        Query parameters
   * @return Return a Post
   */
  public function getDossierPost($collection,$dossier,$post,$ids=null,$iso) {
    $endpoint = $this->apiHostname."collection/".$collection."/dossier/".$dossier."/post/".$post;
    $query = http_build_query(array('apikey'=>$this->apiKey,'apitoken'=>$this->apiToken));
    if(isset($ids))
      $query.= "&ids=".$ids;

    $query.="&iso=".$iso;
    $url = $endpoint."?".$query;

    return $this->restRequest($url);
  }

    /**
   * Get post data
   * @param  string $ids        Query parameters
   * @return Return a post by slug
   */
  public function getSlugPost($slug,$ids=null,$iso) {
    $endpoint = $this->apiHostname."slug/".$slug;
    $query = http_build_query(array('apikey'=>$this->apiKey,'apitoken'=>$this->apiToken));
    if(isset($ids))
      $query.= "&ids=".$ids;

    $query.="&iso=".$iso;
    $url = $endpoint."?".$query;

    return $this->restRequest($url);
  }

  /**
   * Get Collections latest posts
   * @param  int $collection    Collection id
   * @param  string $ids        Query parameters
   * @return Return a Post latest list for a Collection
   */
  public function getCollectionLatestPosts($collection,$perPage=null,$page=null,$ids=null,$iso) {
    $endpoint = $this->apiHostname."collection/".$collection."/latest";
    $query = http_build_query(array('apikey'=>$this->apiKey,'apitoken'=>$this->apiToken));
    if(isset($ids))
      $query.= "&ids=".$ids;

    if(isset($perPage) && isset($page))
      $query.= "&per_page=".$perPage."&page=".$page;
    else 
      if(isset($perPage)) $query.= "&per_page=".$perPage;
      else if(isset($page)) $query.= "&page=".$page;

    $query.="&iso=".$iso;
    $url = $endpoint."?".$query;

    return $this->restRequest($url);
  }


  /**
   * Get Rest request
   * @param  string $url url link
   * @return json      All data from the api take the form
   *                   {"data":{<DATA_RETURNED_FROM_SERVER>}}
   */
  private function restRequest($url) {
    $ch = curl_init();
    curl_setopt ($ch, CURLOPT_URL,$url);
    curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_HEADER, 1);

    $contents = curl_exec($ch);
    $httpCode = curl_getinfo ( $ch, CURLINFO_HTTP_CODE );
    $headerSize = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
    curl_close($ch);

    $header = substr($contents, 0, $headerSize);
    if($httpCode == 304){
        $body = file_get_contents($url);
    }else{
        $body = substr($contents, $headerSize);
    }

    $headerArray = $this->getHeadersFromCurlResponse($header);

    $jdBody = json_decode($body);
    if(!isset($jdBody))
      $jdBody = new stdClass();
    $jdBody->eTag = isset($headerArray["ETag"]) ? $headerArray["ETag"] : "";
   
    $return = $jdBody;

    if($httpCode != 200 && $httpCode != 304) {
      $return = null;
    }
    

    return $return;
  }

  /* Get array of Headers From Curl Response */
  private function getHeadersFromCurlResponse($response) {
    $headers = array();

    $header_text = substr($response, 0, strpos($response, "\r\n\r\n"));

    foreach (explode("\r\n", $header_text) as $i => $line) {
      if ($i === 0)
        $headers['http_code'] = $line;
      else {
        list ($key, $value) = explode(': ', $line);
        $headers[$key] = $value;
      }
    }
    return $headers;
  }

  /***************************/
  /***************************/


  /* Post Api - Utils Methods */
  /***************************/


  /**
   * Get all tags from collection
   * @param  int      $collection collection id
   * @param  string   $lang       current language
   * @return array of info tags
   */
  public function getCollectionTags($collection,$lang) {
    $resCollection = $this->getCollection($collection);

    $tags = array();
    if(!isset($resCollection->error) && isset($resCollection->data->tags)) {
      foreach ($resCollection->data->tags as $key => $cTags) {
        if($cTags->iso == $lang) 
          return $cTags->list;
      }
    }
    return $tags;
  }

  /**
   * Get all tags from dossier
   * @param  int $collection collection id
   * @param  int $dossier    dossier id
   * @return array of info tags
   */
  public function getDossierTags($collection,$dossier) {
    $resCollection = $this->getCollectionDossier($collection,$dossier);

    $tags = array();
    if(!isset($resCollection->error)) {
      return $resCollection->data->tags;
    }
    return $tags;
  }
 
}